<?php
namespace Teknisa\MAN\Controllers;

use Zeedhi\Framework\Controller\CrudWithParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use \Zeedhi\Framework\DTO\Request\Row;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Notification;
use Teknisa\MAN\Utilities;
use Teknisa\MAN\Services\ParaGeraisProdutoService;

class ParaGeraisProduto extends CrudWithParameterBag {

    protected $parameterBag;
    protected $dataSourceNames = array (
        'filterProduto'         => 'paraGeraisProduto',
        'filterPercCalcProduto' => 'paraGeraisPercCalcPrecProduto',
        'selectProdutosPercent' => 'vnd10003_paramGerais_select_products'
    );
    protected $parameterBagColumnsProduto         = array('NRORG', 'CDARVPROD', 'CDFILIAL');
    protected $parameterBagColumnsPercCalcProduto = array('NRORG', 'CDFILIAL');
    protected $paraGeraisProdutoService;

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, ParaGeraisProdutoService $paraGeraisProdutoService,Utilities\Util $utilService) {
        parent::__construct($dataSourceManager, $parameterBag);
        $this->paraGeraisProdutoService = $paraGeraisProdutoService;
        $this->utilService = $utilService;
    }

    public function findProduto(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNames['filterProduto'];

        $this->parameterBagColumns = $this->parameterBagColumnsProduto;
        $conditions = $this->utilService->getParamsAsArray($request);

        if(!array_key_exists('CDARVPROD',$conditions)){
             $conditions['CDARVPROD'] = null;
             $dataSourceFilter = array();

            foreach($conditions as $key => $value) {
                $dataSourceFilter[] = array(
                    "columnName" => $key,
                    "operator"=> "=",
                    "value"=> $value
                );
            }

            $original = $request->getFilterCriteria()->getConditions();

            for($i = 0 ; $i < count($dataSourceFilter) ; $i++) {
                for($j = 0 ; $j < count($original) ; $j++ ) {
                    if( $dataSourceFilter[$i]['columnName'] === $original[$j]['columnName'] &&
                        $dataSourceFilter[$i]['operator'] !== $original[$j]['operator'])  {
                        $dataSourceFilter[$i]['operator'] = $original[$j]['operator'];
                    }
                }
            }
        } else {
            $dataSourceFilter = $request->getFilterCriteria()->getConditions();
            $dataSourceFilter[2]["operator"] = "=";
            $dataSourceFilter[2]["value"] = str_replace('%', '', $dataSourceFilter[2]["value"]);
        }

        $newRequest = new DTO\Request\Filter(
            new FilterCriteria(
                $request->getFilterCriteria()->getDataSourceName(),
                $dataSourceFilter,
                $request->getFilterCriteria()->getPage(),
                $request->getFilterCriteria()->getPageSize()
            ),
            $request->getMethod(),
            $request->getRoutePath(),
            $request->getUserId()
        );
        parent::find($newRequest, $response);
    }

    public function findProdutoFilter(DTO\Request\Filter $request, DTO\Response $response){
	   	try{
   			$row = $request->getFilterCriteria()->getConditions();
   			$NRORG = $row[0];
            $CDARVPROD = $row[1];
            $result = $this->paraGeraisProdutoService->findProdutoField($NRORG, $CDARVPROD);
            $response->addDataSet(new DataSet('paraGeraisProdutoFilter', $result));
        }
        catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }

    public function saveProduto(DTO\Request\DataSet $request, DTO\Response $response) {
        $row = $request->getDataSet()->getRows();
        $NRORG               = $row[0]['NRORG'];
        $CDPRODUTO           = $row[0]['CDPRODUTO'];
        $IDPESAPROD          = $row[0]['IDPESAPROD'];
        $QTTARAPROD          = $row[0]['QTTARAPROD'];
        $CDBARPRODUTO        = $row[0]['CDBARPRODUTO'];
        $NMPRODIMPFIS        = $row[0]['NMPRODIMPFIS'];
        $IDCOBTXSERV         = $row[0]['IDCOBTXSERV'];
        $IDTIPOPROD          = $row[0]['IDTIPOPROD'];
        $CDPROTECLADO        = $row[0]['CDPROTECLADO'];
        $IDIMPPRODUTO        = $row[0]['IDIMPPRODUTO'];
        $IDIMPRODUVEZ        = $row[0]['IDIMPRODUVEZ'];
        $IDSOLLOCIMP         = $row[0]['IDSOLLOCIMP'];
        $DSDADOSFORNVND      = $row[0]['DSDADOSFORNVND'];
        $QTDIAVALPRODVND     = $row[0]['QTDIAVALPRODVND'];
        $DSINFADIPRODVND     = $row[0]['DSINFADIPRODVND'];
        $QTPORCAOVND         = $row[0]['QTPORCAOVND'];
        $IDUNPORCAOVND       = $row[0]['IDUNPORCAOVND'];
        $IDPADEMEDCASVND     = $row[0]['IDPADEMEDCASVND'];
        $IDTIPOMEDCASVND     = $row[0]['IDTIPOMEDCASVND'];
        $QTMEDCASVND         = $row[0]['QTMEDCASVND'];
        $IDORIGPRODPR        = $row[0]['IDORIGPRODPR'];
        $QTMAXOBSERV         = $row[0]['QTMAXOBSERV'];

        $this->paraGeraisProdutoService->updateProduto($NRORG, $CDPRODUTO, $IDPESAPROD, $QTTARAPROD, $CDBARPRODUTO, $NMPRODIMPFIS, $IDCOBTXSERV, $IDTIPOPROD, $CDPROTECLADO, $IDIMPPRODUTO, $IDIMPRODUVEZ, $IDSOLLOCIMP, $DSDADOSFORNVND, $QTDIAVALPRODVND, $DSINFADIPRODVND, $QTPORCAOVND, $IDUNPORCAOVND, $IDPADEMEDCASVND, $IDTIPOMEDCASVND, $QTMEDCASVND, $IDORIGPRODPR, $QTMAXOBSERV);
        $response->addNotification(new Notification("Produto alterado com sucesso!", Notification::TYPE_SUCCESS));
    }

    public function findPercCalcProduto(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNames['filterPercCalcProduto'];
        $this->parameterBagColumns = $this->parameterBagColumnsPercCalcProduto;
        parent::find($request, $response);
    }

    public function saveProdPerc($request, $response) {
	   	try {
            $row = $request->getDataSet()->getRows()[0];

            $CDARVPRODINI   = $row['CDARVPRODINI'];
            $CDARVPRODFIN   = $row['CDARVPRODFIN'];
            $PRODUTOS       = $row['PRODUTOS'];
            $CDFILIAL       = $row['CDFILIAL'];
            $VRPERCPREPRO   = $row['VRPERCPREPRO'];
            $NRORG          = $row['NRORG'];
            $save = $this->paraGeraisProdutoService->saveProdutos($CDARVPRODINI, $CDARVPRODFIN, $PRODUTOS,
                                                           $CDFILIAL, $VRPERCPREPRO, $NRORG);
            } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
	}

    public function findProductsPercentual(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNames['selectProdutosPercent'];
        $this->parameterBagColumns = $this->parameterBagColumnsPercCalcProduto;
        parent::find($request, $response);
    }



	public function deleteProdutoPercentual(Row $params, Response $response) {
	    $array = $params->getRow();
        $NRORG               = $array['NRORG'];
        $CDPRODUTO           = $array['CDPRODUTO'];
        $CDFILIAL            = $array['CDFILIAL'];
        $this->paraGeraisProdutoService->deletePercCalcProduto($CDFILIAL, $NRORG, $CDPRODUTO);
    }

    private function createFilterCriteria($params) {
        $filterCriteria = new FilterCriteria ($this->dataSourceName);

        foreach($params as $name => $value) {
            $filterCriteria->addCondition($name, '=', $value);
        }

        return $filterCriteria;
    }

    public function save (DTO\Request\DataSet $request, DTO\Response $response){
        $this->dataSourceName = $this->dataSourceNames['filterProduto'];
        $this->parameterBagColumns = array('CDPRODUTO','NRORG');
        $row = $request->getDataSet()->getRows();
        foreach($request->getDataSet()->getRows() as $key => &$row){
            if($row['IDIMPPRODUTO']== null) {
                $row['IDIMPPRODUTO'] = "1";
            }


        }
        $this->saveCrud($request,$response);
    }

    public function saveCrud(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $dataSet = $request->getDataSet();
            $dataSet->setDataSourceName($this->dataSourceName);
            $persistedRows = $this->dataSourceManager->persist($dataSet);
            $response->addDataSet(new DataSet($this->dataSourceName, $persistedRows));
            $nrPersistedRows = count($persistedRows);
        } catch (\Exception $e) {
            $response->setCriticalError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }
}